@@ -11,9 +11,7 @@ an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express o
 specific language governing permissions and limitations under the License.
 """
 
-"""
-引擎核心模块
-"""
+# 引擎核心模块
 
 import time
 import random
@@ -42,8 +40,6 @@ from .eri import (
     EngineRuntimeInterface,
     ScheduleType,
     NodeType,
-    NodeType,
-    ProcessInfo,
     State,
     ExecutionData,
     DataInput,
@@ -228,8 +224,7 @@ class Engine:
 
         if data is not None:
             self.runtime.set_data_inputs(
-                node_id,
-                {k: DataInput(need_render=True, value=v) for k, v in data.items()},
+                node_id, {k: DataInput(need_render=True, value=v) for k, v in data.items()},
             )
 
         self._add_history(node_id, state)
@@ -257,7 +252,7 @@ class Engine:
         """
         node = self.runtime.get_node(node_id)
 
-        if not node.type is NodeType.SubProcess:
+        if node.type is not NodeType.SubProcess:
             raise InvalidOperationError("node({}) type is not SubProcess".format(node_id))
 
         state = self.runtime.get_state(node_id)
@@ -316,11 +311,7 @@ class Engine:
         self._add_history(node_id, state)
 
         self.runtime.set_state(
-            node_id=node_id,
-            to_state=states.FINISHED,
-            is_skip=True,
-            refresh_version=True,
-            set_archive_time=True,
+            node_id=node_id, to_state=states.FINISHED, is_skip=True, refresh_version=True, set_archive_time=True,
         )
 
         # 跳过节点时不再做节点输出提取到上下文的操作
@@ -355,11 +346,7 @@ class Engine:
         self._add_history(node_id, state)
 
         self.runtime.set_state(
-            node_id=node_id,
-            to_state=states.FINISHED,
-            is_skip=True,
-            refresh_version=True,
-            set_archive_time=True,
+            node_id=node_id, to_state=states.FINISHED, is_skip=True, refresh_version=True, set_archive_time=True,
         )
 
         self.runtime.execute(process_id, next_node_id)
@@ -401,10 +388,7 @@ class Engine:
 
         old_ver = state.version
         new_ver = self.runtime.set_state(
-            node_id=node_id,
-            to_state=states.FAILED,
-            refresh_version=True,
-            set_archive_time=True,
+            node_id=node_id, to_state=states.FAILED, refresh_version=True, set_archive_time=True,
         )
 
         self.runtime.set_execution_data_outputs(node_id, outputs)
@@ -504,9 +488,7 @@ class Engine:
                 if node_state_map[process_info.root_pipeline_id] == states.REVOKED:
                     self.runtime.die(process_id)
                     logger.info(
-                        "[%s] root pipeline revoked checked at node %s",
-                        process_info.root_pipeline_id,
-                        current_node_id,
+                        "[%s] root pipeline revoked checked at node %s", process_info.root_pipeline_id, current_node_id,
                     )
                     return
 
@@ -541,9 +523,7 @@ class Engine:
 
                         self.runtime.set_execution_data_outputs(current_node_id, exec_outputs)
                         self.runtime.set_state(
-                            node_id=current_node_id,
-                            to_state=states.FAILED,
-                            set_archive_time=True,
+                            node_id=current_node_id, to_state=states.FAILED, set_archive_time=True,
                         )
                         self.runtime.sleep(process_id)
 
@@ -655,6 +635,7 @@ class Engine:
                     ex_data,
                 )
 
+                # state version already changed, so give up this execute
                 if isinstance(e, StateVersionNotMatchError):
                     logger.warning(
                         "[%s]execute exception catch StateVersionNotMatchError at node(%s): %s",
@@ -685,11 +666,7 @@ class Engine:
     @setup_gauge(ENGINE_RUNNING_SCHEDULES)
     @setup_histogram(ENGINE_SCHEDULE_RUNNING_TIME)
     def schedule(
-        self,
-        process_id: int,
-        node_id: str,
-        schedule_id: str,
-        callback_data_id: Optional[int] = None,
+        self, process_id: int, node_id: str, schedule_id: str, callback_data_id: Optional[int] = None,
     ):
         """
         在某个进程上开始某个节点的调度
@@ -708,10 +685,64 @@ class Engine:
             process_info = self.runtime.get_process_info(process_id)
             root_pipeline_id = process_info.root_pipeline_id
 
+            state = self.runtime.get_state(node_id)
+            schedule = self.runtime.get_schedule(schedule_id)
+
+            # engine context prepare
+            set_node_info(CurrentNodeInfo(node_id=node_id, version=state.version, loop=state.loop))
+
+            # schedule alredy finished
+            if schedule.finished:
+                logger.warning(
+                    "[%s] schedule(%s) %s with version %s already finished",
+                    root_pipeline_id,
+                    schedule_id,
+                    node_id,
+                    schedule.version,
+                )
+                return
+
+            # 检查 schedule 是否过期
+            if state.version != schedule.version:
+                logger.info(
+                    "[%s] schedule(%s) %s with version %s expired, current version: %s",
+                    root_pipeline_id,
+                    schedule_id,
+                    node_id,
+                    schedule.version,
+                    state.version,
+                )
+                self.runtime.expire_schedule(schedule_id)
+                return
+
+            # 检查节点状态是否合法
+            if state.name != states.RUNNING:
+                logger.info(
+                    "[%s] schedule(%s) %s with version %s state is not running: %s",
+                    root_pipeline_id,
+                    schedule_id,
+                    node_id,
+                    schedule.version,
+                    state.name,
+                )
+                self.runtime.expire_schedule(schedule_id)
+                return
+
             # try to get lock
             lock_get = self.runtime.apply_schedule_lock(schedule_id)
 
             if not lock_get:
+                # only retry at multiple calback type
+                if schedule.type is not ScheduleType.MULTIPLE_CALLBACK:
+                    logger.info(
+                        "[%s] schedule(%s) %s with version %s is not multiple callback type, will not retry to get lock",  # noqa
+                        root_pipeline_id,
+                        schedule_id,
+                        node_id,
+                        schedule.version,
+                    )
+                    return
+
                 try_after = random.randint(1, 5)
                 logger.info(
                     "[%s] schedule(%s) lock %s with data %s fetch fail, try after %s",
@@ -734,41 +765,6 @@ class Engine:
                 # 进程心跳
                 self.runtime.beat(process_id)
 
-                schedule = self.runtime.get_schedule(schedule_id)
-
-                # schedule alredy finished
-                if schedule.finished:
-                    return
-
-                # 检查 schedule 是否过期
-                state = self.runtime.get_state(node_id)
-                if state.version != schedule.version:
-                    logger.info(
-                        "[%s] schedule(%s) %s with version %s expired, current version: %s",
-                        root_pipeline_id,
-                        schedule_id,
-                        node_id,
-                        schedule.version,
-                        state.version,
-                    )
-                    self.runtime.expire_schedule(schedule_id)
-                    return
-
-                # 检查节点状态是否合法
-                if state.name != states.RUNNING:
-                    logger.info(
-                        "[%s] schedule(%s) %s state is not running: %s",
-                        root_pipeline_id,
-                        schedule_id,
-                        node_id,
-                        state.name,
-                    )
-                    self.runtime.expire_schedule(schedule_id)
-                    return
-
-                # engine context prepare
-                set_node_info(CurrentNodeInfo(node_id=node_id, version=state.version, loop=state.loop))
-
                 # fetch callback data
                 callback_data = None
                 if callback_data_id:
@@ -780,10 +776,7 @@ class Engine:
                 type_label = self._get_metrics_node_type(node)
 
                 logger.info(
-                    "[%s] before schedule node %s with data %s",
-                    root_pipeline_id,
-                    node,
-                    callback_data,
+                    "[%s] before schedule node %s with data %s", root_pipeline_id, node, callback_data,
                 )
                 schedule_start = time.time()
                 schedule_result = handler.schedule(process_info, state.loop, state.inner_loop, schedule, callback_data)
@@ -791,10 +784,7 @@ class Engine:
 
                 if schedule_result.has_next_schedule:
                     self.runtime.set_next_schedule(
-                        process_info.process_id,
-                        node_id,
-                        schedule_id,
-                        schedule_result.schedule_after,
+                        process_info.process_id, node_id, schedule_id, schedule_result.schedule_after,
                     )
 
                 if schedule_result.schedule_done:
@@ -803,12 +793,10 @@ class Engine:
         except Exception as e:
             ex_data = traceback.format_exc()
             logger.warning(
-                "[%s]schedule exception catch at node(%s): %s",
-                root_pipeline_id,
-                node_id,
-                ex_data,
+                "[%s]schedule exception catch at node(%s): %s", root_pipeline_id, node_id, ex_data,
             )
 
+            # state version already changed, so give up this schedule
             if isinstance(e, StateVersionNotMatchError):
                 logger.exception(
                     "[%s]schedule exception catch StateVersionNotMatchError at node(%s): %s",
@@ -835,10 +823,7 @@ class Engine:
         self.runtime.release_schedule_lock(schedule_id)
 
     def _add_history(
-        self,
-        node_id: str,
-        state: Optional[State] = None,
-        exec_data: Optional[ExecutionData] = None,
+        self, node_id: str, state: Optional[State] = None, exec_data: Optional[ExecutionData] = None,
     ) -> int:
         if not state:
             state = self.runtime.get_state(node_id)
