@@ -298,6 +298,7 @@ class State(Representable):
         name: str,
         version: str,
         loop: int,
+        inner_loop: int,
         retry: int,
         skip: bool,
         error_ignored: bool,
@@ -318,6 +319,8 @@ class State(Representable):
         :type version: str
         :param loop: 重入次数
         :type loop: int
+        :param inner_loop: 子流程重入次数
+        :type inner_loop: int
         :param retry: 重试次数
         :type retry: int
         :param skip: 是否跳过
@@ -337,6 +340,7 @@ class State(Representable):
         self.name = name
         self.version = version
         self.loop = loop
+        self.inner_loop = inner_loop
         self.retry = retry
         self.skip = skip
         self.error_ignored = error_ignored
